/**
 * (C) Le Hong Phuong, phuonglh@gmail.com
 *  Vietnam National University, Hanoi, Vietnam.
 */
package vn.hus.nlp.tokenizer.segmenter;

import java.io.File;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.vietspider.common.io.LogService;
import org.vietspider.common.io.UtilFile;

import vn.hus.nlp.lexicon.LexiconUnmarshaller;
import vn.hus.nlp.utils.UTF8FileUtility;

/**
 * @author Le Hong Phuong, phuonglh@gmail.com
 * <p>
 * vn.hus.nlp.segmenter
 * <p>
 * Nov 15, 2007, 11:51:08 PM
 * <p>
 * An accent normalizer for Vietnamese string. The purpose of
 * this class is to convert a syllable like "hòa" to "hoà",
 * since the lexicon contains only the later form.
 */
public final class StringNormalizer {

  private static Map<String, String> map;

  private StringNormalizer(String mapFile) {
    map = new HashMap<String, String>();
    init(mapFile);
  }


  private void init(String mapFile) {
    try {
//      System.out.println(" =====  >"+ mapFile + " : "+ url);
      File file = UtilFile.getFile("system/nlp/tokenizer", mapFile);
      String[] rules = UTF8FileUtility.getLines(file.getAbsolutePath());
      for (int i = 0; i < rules.length; i++) {
        String[] s = rules[i].split("\\s+");
        if (s.length == 2) {
          map.put(s[0], s[1]);
        } else {
          LogService.getInstance().setMessage(null, 
              "Wrong syntax in the map file " + mapFile + " at line " + i);
        }
      }
    } catch (Exception e) {
      LogService.getInstance().setThrowable(e);
    }
  }


  /**
   * @return an instance of the class.
   */
  public static StringNormalizer getInstance() {
    return new StringNormalizer(IConstants.NORMALIZATION_RULES);
  }

  /**
   * Normalize a string.
   * @return a normalized string
   * @param s a string
   */
  public String normalize(String s) {
    String result = new String(s);
    for (Iterator<String> it = map.keySet().iterator(); it.hasNext();) {
      String from = it.next();
      String to = map.get(from);
      if (result.indexOf(from) >= 0) {
        result = result.replace(from, to);
      }
    }
    return result;
  }

}
