package vn.hus.nlp.utils;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class CaseConverter
{
  public static CaseConverter caseConverter = new CaseConverter();
  private static Map<Character, Character> lower2UpperMap;
  private static Map<Character, Character> upper2LowerMap;
  private String[] lowerCharacters = { "1b0", "1ecb", "1ef1", "1ea7", "1ee7", "1edd", "1eb9", "1eb7", "1ef7", "e1", "1ed7", "1eed", "e0", "129", "1ecd", "1ea5", "ea", "1eeb", "111", "1ecf", "fa", "1ed9", "e2", "1ebd", "f5", "1ea3", "103", "1ee9", "1ec9", "1ea9", "1eb3", "1edb", "f9", "e8", "e3", "1ed5", "1eaf", "1eef", "1ef9", "1ec3", "f4", "1ead", "169", "e9", "1ee5", "f3", "fd", "1ec5", "1a1", "1ebf", "1edf", "1eb5", "1ef5", "1eab", "f2", "1ed1", "1ec7", "1ee3", "ec", "ed", "1ea1", "1ef3", "1ed3", "1eb1", "1ebb", "1ec1", "1ee1" };
  private String[] upperCharacters = { "1af", "1eca", "1ef0", "1ea6", "1ee6", "1edc", "1eb8", "1eb6", "1ef6", "c1", "1ed6", "1eec", "c0", "128", "1ecc", "1ea4", "ca", "1eea", "110", "1ece", "da", "1ed8", "c2", "1ebc", "d5", "1ea2", "102", "1ee8", "1ec8", "1ea8", "1eb2", "1eda", "d9", "c8", "c3", "1ed4", "1eae", "1eee", "1ef8", "1ec2", "d4", "1eac", "168", "c9", "1ee4", "d3", "dd", "1ec4", "1a0", "1ebe", "1ede", "1eb4", "1ef4", "1eaa", "d2", "1ed0", "1ec6", "1ee2", "cc", "cd", "1ea0", "1ef2", "1ed2", "1eb0", "1eba", "1ec0", "1ee0" };

  private CaseConverter()
  {
    lower2UpperMap = new HashMap();
    upper2LowerMap = new HashMap();
    init();
  }

  public static boolean isValidLower(char paramChar)
  {
    Character localCharacter = new Character(paramChar);
    return lower2UpperMap.containsKey(localCharacter);
  }

  public static boolean isValidUpper(char paramChar)
  {
    Character localCharacter = new Character(paramChar);
    return upper2LowerMap.containsKey(localCharacter);
  }

  public static char toUpper(char paramChar)
  {
    Character localCharacter = (Character)lower2UpperMap.get(new Character(paramChar));
    return localCharacter.charValue();
  }

  public static char toLower(char paramChar)
  {
    Character localCharacter = (Character)upper2LowerMap.get(new Character(paramChar));
    return localCharacter.charValue();
  }

  public static String toLower(String paramString)
  {
    StringBuffer localStringBuffer = new StringBuffer(paramString);
    for (int i = 0; i < paramString.length(); i++)
    {
      char c1 = paramString.charAt(i);
      char c2 = c1;
      if (('A' <= c1) && (c1 <= 'Z'))
        c2 = Character.toLowerCase(c1);
      else if (isValidUpper(c1))
        c2 = toLower(c1);
      localStringBuffer.setCharAt(i, c2);
    }
    return localStringBuffer.toString();
  }

  private void init()
  {
    for (int i = 0; i < this.lowerCharacters.length; i++)
    {
      String str1 = this.lowerCharacters[i];
      String str2 = this.upperCharacters[i];
      Character localCharacter1 = new Character((char)Integer.parseInt(str1, 16));
      Character localCharacter2 = new Character((char)Integer.parseInt(str2, 16));
      lower2UpperMap.put(localCharacter1, localCharacter2);
      upper2LowerMap.put(localCharacter2, localCharacter1);
    }
  }

  public void convert(String paramString)
    throws IOException
  {
    FileOutputStream localFileOutputStream = new FileOutputStream(paramString);
    OutputStreamWriter localOutputStreamWriter = new OutputStreamWriter(localFileOutputStream, "UTF-8");
    BufferedWriter localBufferedWriter = new BufferedWriter(localOutputStreamWriter);
    Iterator localIterator = lower2UpperMap.keySet().iterator();
    while (localIterator.hasNext())
    {
      Character localCharacter1 = (Character)localIterator.next();
      Character localCharacter2 = (Character)lower2UpperMap.get(localCharacter1);
      localBufferedWriter.write(localCharacter1.toString());
      localBufferedWriter.write("\t");
      localBufferedWriter.write(localCharacter2.toString());
      localBufferedWriter.write("\n");
    }
    localBufferedWriter.close();
  }

  public static boolean containsUppercase(String paramString)
  {
    for (int i = 0; i < paramString.length(); i++)
    {
      char c = paramString.charAt(i);
      if ((Character.isUpperCase(c)) || (isValidUpper(c)))
        return true;
    }
    return false;
  }
}

/* Location:           C:\Downloads\vn.hus.nlp.tokenizer-4.1.1-bin.tar\lib\vn.hus.nlp.utils-1.0.0.jar
 * Qualified Name:     vn.hus.nlp.utils.CaseConverter
 * JD-Core Version:    0.6.0
 */